package de.letsbuildacompiler.compiler;

import de.letsbuildacompiler.compiler.exceptions.FunctionAlreadyDefinedException;
import de.letsbuildacompiler.parser.DemoBaseVisitor;
import de.letsbuildacompiler.parser.DemoParser;
import org.antlr.v4.runtime.tree.ParseTree;

import java.util.HashSet;
import java.util.Set;

public class FunctionDefinitionFinder {

    public static FunctionList findFunctions(ParseTree tree) {
        final FunctionList definedFunctions = new FunctionList();

        new DemoBaseVisitor<Void>() {
            @Override
            public Void visitFunctionDefinition(DemoParser.FunctionDefinitionContext ctx) {
                String functionName = ctx.funcName.getText();
                int numberOfArguments = ctx.params.declarations.size();
                if (definedFunctions.contains(functionName, numberOfArguments)) {
                    throw new FunctionAlreadyDefinedException(ctx.funcName);
                }
                definedFunctions.add(functionName, numberOfArguments);
                return null;
            }
        }.visit(tree);

        return definedFunctions;
    }
}
